@@ -4,8 +4,6 @@ import tempfile
 import textwrap
 
 import pytest
-import six
-from six.moves import range
 
 from testing.testifycompat import (
     assert_equal,
@@ -17,16 +15,14 @@ from staticconf import loader, errors
 
 
 def get_bytecode_filename(module_name):
-    if six.PY2:
-        return module_name + '.pyc'
     return __import__(module_name).__cached__
 
 
-class LoaderTestCase(object):
+class LoaderTestCase:
 
     content = None
 
-    @pytest.yield_fixture(autouse=True)
+    @pytest.fixture(autouse=True)
     def mock_config(self):
         with mock.patch('staticconf.config') as self.mock_config:
             yield
@@ -130,7 +126,7 @@ class TestAutoConfiguration(LoaderTestCase):
     def setup_filename(self):
         self.filename = None
 
-    @pytest.yield_fixture(autouse=True)
+    @pytest.fixture(autouse=True)
     def cleanup_file(self):
         yield
         if self.filename:
@@ -169,7 +165,7 @@ class TestPythonConfiguration(LoaderTestCase):
         }
     """)
 
-    @pytest.yield_fixture(autouse=True)
+    @pytest.fixture(autouse=True)
     def teardown_module(self):
         yield
         self.remove_module()
@@ -304,7 +300,7 @@ class TestPropertiesConfiguration(LoaderTestCase):
         assert_equal(config_data['key.with.col'], 'a value')
 
     def test_invalid_line(self):
-        self.tmpfile.write('justkey\n'.encode('utf8'))
+        self.tmpfile.write(b'justkey\n')
         self.tmpfile.flush()
         assert_raises(
                 errors.ConfigurationError,
@@ -312,7 +308,7 @@ class TestPropertiesConfiguration(LoaderTestCase):
                 self.tmpfile.name)
 
 
-class TestCompositeConfiguration(object):
+class TestCompositeConfiguration:
 
     def test_load(self):
         loaders = [(mock.Mock(return_value={i: 0}), 1, 2) for i in range(3)]
@@ -323,7 +319,7 @@ class TestCompositeConfiguration(object):
             loader_call.assert_called_with(arg_one, arg_two)
 
 
-class StubObject(object):
+class StubObject:
     year = 2012
     month = 3
     hour = 15
